@@ -76,7 +76,7 @@ ev_module_load (GTypeModule *gmodule)
         module->library = g_module_open (module->path, 0);
 
         if (!module->library) {
-                g_warning (g_module_error ());
+                g_warning ("%s", g_module_error ());
 
                 return FALSE;
         }
@@ -84,7 +84,7 @@ ev_module_load (GTypeModule *gmodule)
         /* extract symbols from the lib */
         if (!g_module_symbol (module->library, "register_evince_backend",
                               (void *) &register_func)) {
-                g_warning (g_module_error ());
+                g_warning ("%s", g_module_error ());
                 g_module_close (module->library);
 
                 return FALSE;
