@@ -28,16 +28,22 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
+#ifdef GDK_WINDOWING_X11
 #include <gdk/gdkx.h>
+#endif
 
 #include "totem-scrsaver.h"
+
+#ifdef WITH_SMCLIENT
 #include "eggsmclient.h"
+#endif
 
 #include "ev-application.h"
 #include "ev-document-factory.h"
 #include "ev-file-helpers.h"
 #include "ev-metadata-manager.h"
 #include "ev-utils.h"
+#include "ev-stock-icons.h"
 
 #ifdef ENABLE_DBUS
 #include "ev-media-player-keys.h"
@@ -48,23 +54,31 @@
 #include "ev-application-service.h"
 #endif
 
-static void ev_application_add_icon_path_for_screen (GdkScreen     *screen);
-static void ev_application_save_print_settings      (EvApplication *application);
+static void ev_application_save_print_settings (EvApplication *application);
 
 struct _EvApplication {
 	GObject base_instance;
 
 	gchar *dot_dir;
+	gchar *data_dir;
 	gchar *accel_map_file;
 	gchar *toolbars_file;
 
+#ifdef ENABLE_DBUS
+	gchar *crashed_file;
+	guint  crashed_idle;
+#endif
+
 	EggToolbarsModel *toolbars_model;
 
 	TotemScrsaver *scr_saver;
 
+#ifdef WITH_SMCLIENT
 	EggSMClient *smclient;
+#endif
 
-	gchar *last_chooser_uri;
+	gchar *filechooser_open_uri;
+	gchar *filechooser_save_uri;
 
 #ifdef ENABLE_DBUS
 	EvMediaPlayerKeys *keys;
@@ -161,18 +175,190 @@ ev_application_get_instance (void)
 	return instance;
 }
 
+#if defined (WITH_SMCLIENT) || defined (ENABLE_DBUS)
+
 /* Session */
+static void
+save_session (EvApplication *application,
+	      GList         *windows_list,
+	      GKeyFile      *state_file)
+{
+	GList *l;
+	gint i;
+	const gchar **uri_list;
+	const gchar *empty = "empty-window";
+
+	uri_list = g_new (const gchar *, g_list_length (windows_list));
+	for (l = windows_list, i = 0; l != NULL; l = g_list_next (l), i++) {
+		EvWindow *window = EV_WINDOW (l->data);
+
+		if (ev_window_is_empty (window))
+			uri_list[i] = empty;
+		else
+			uri_list[i] = ev_window_get_uri (window);
+	}
+	g_key_file_set_string_list (state_file,
+				    "Evince",
+				    "documents",
+				    (const char **)uri_list,
+				    i);
+	g_free (uri_list);
+}
+
+#endif /* WITH_SMCLIENT || ENABLE_DBUS */
+
+#ifdef ENABLE_DBUS
+static void
+ev_application_save_session_crashed (EvApplication *application)
+{
+	GList *windows;
+
+	windows = ev_application_get_windows (application);
+	if (windows) {
+		GKeyFile *crashed_file;
+		gchar    *data;
+		gssize    data_length;
+		GError   *error = NULL;
+
+		crashed_file = g_key_file_new ();
+		save_session (application, windows, crashed_file);
+
+		data = g_key_file_to_data (crashed_file, (gsize *)&data_length, NULL);
+		g_file_set_contents (application->crashed_file, data, data_length, &error);
+		if (error) {
+			g_warning ("%s", error->message);
+			g_error_free (error);
+		}
+		g_free (data);
+		g_key_file_free (crashed_file);
+	} else if (g_file_test (application->crashed_file, G_FILE_TEST_IS_REGULAR)) {
+		GFile *file;
+
+		file = g_file_new_for_path (application->crashed_file);
+		g_file_delete (file, NULL, NULL);
+		g_object_unref (file);
+	}
+}
+
+static gboolean
+save_session_crashed_in_idle_cb (EvApplication *application)
+{
+	ev_application_save_session_crashed (application);
+	application->crashed_idle = 0;
+
+	return FALSE;
+}
+
+static void
+save_session_crashed_in_idle (EvApplication *application)
+{
+	if (application->crashed_idle > 0)
+		g_source_remove (application->crashed_idle);
+	application->crashed_idle =
+		g_idle_add ((GSourceFunc)save_session_crashed_in_idle_cb,
+			    application);
+}
+
+static gboolean
+ev_application_run_crash_recovery_dialog (EvApplication *application,
+					  const gchar  **files)
+{
+	GtkWidget *dialog;
+	gint       response;
+
+	/* Do not show the recover dialog if the requested file is the
+	 * only one to be recovered
+	 */
+	if (files && g_strv_length ((gchar **)files) == 1) {
+		GKeyFile *state_file;
+		gchar   **uri_list;
+
+		state_file = g_key_file_new ();
+		g_key_file_load_from_file (state_file,
+					   application->crashed_file,
+					   G_KEY_FILE_NONE,
+					   NULL);
+		uri_list = g_key_file_get_string_list (state_file,
+						       "Evince",
+						       "documents",
+						       NULL, NULL);
+		if (uri_list && g_strv_length (uri_list) == 1) {
+			GFile *file;
+			gchar *uri;
+
+			file = g_file_new_for_commandline_arg (files[0]);
+			uri = g_file_get_uri (file);
+			g_object_unref (file);
+			if (g_ascii_strcasecmp (uri, uri_list[0]) == 0) {
+				g_strfreev (uri_list);
+				g_key_file_free (state_file);
+				g_free (uri);
+
+				return FALSE;
+			}
+			g_free (uri);
+			g_strfreev (uri_list);
+		}
+		g_key_file_free (state_file);
+	}
+
+	dialog = gtk_message_dialog_new	(NULL,
+					 GTK_DIALOG_MODAL,
+					 GTK_MESSAGE_WARNING,
+					 GTK_BUTTONS_NONE,
+					 _("Recover previous documents?"));
+	gtk_message_dialog_format_secondary_text (
+		GTK_MESSAGE_DIALOG (dialog),
+		_("Evince appears to have exited unexpectedly the last time "
+		  "it was run. You can recover the opened documents."));
+
+	gtk_dialog_add_button (GTK_DIALOG (dialog),
+			       _("_Don't Recover"),
+			       GTK_RESPONSE_CANCEL);
+	gtk_dialog_add_button (GTK_DIALOG (dialog),
+			       _("_Recover"),
+			       GTK_RESPONSE_ACCEPT);
+
+	gtk_window_set_title (GTK_WINDOW (dialog), _("Crash Recovery"));
+	gtk_window_set_icon_name (GTK_WINDOW (dialog), "evince");
+	gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
+	gtk_window_set_skip_taskbar_hint (GTK_WINDOW (dialog), FALSE);
+	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT);
+
+	response = gtk_dialog_run (GTK_DIALOG (dialog));
+	gtk_widget_destroy (dialog);
+
+	return response == GTK_RESPONSE_ACCEPT;
+}
+#endif /* ENABLE_DBUS */
+
 gboolean
-ev_application_load_session (EvApplication *application)
+ev_application_load_session (EvApplication *application,
+			     const gchar  **files)
 {
 	GKeyFile *state_file;
 	gchar   **uri_list;
-	
-	if (!egg_sm_client_is_resumed (application->smclient))
-		return FALSE;
 
-	state_file = egg_sm_client_get_state_file (application->smclient);
-	if (!state_file)
+#ifdef WITH_SMCLIENT
+	if (egg_sm_client_is_resumed (application->smclient)) {
+		state_file = egg_sm_client_get_state_file (application->smclient);
+		if (!state_file)
+			return FALSE;
+	} else
+#endif /* WITH_SMCLIENT */
+#ifdef ENABLE_DBUS
+        if (g_file_test (application->crashed_file, G_FILE_TEST_IS_REGULAR)) {
+		if (ev_application_run_crash_recovery_dialog (application, files)) {
+			state_file = g_key_file_new ();
+			g_key_file_load_from_file (state_file,
+						   application->crashed_file,
+						   G_KEY_FILE_NONE,
+						   NULL);
+		} else {
+			return FALSE;
+		}
+	} else
+#endif /* ENABLE_DBUS */
 		return FALSE;
 
 	uri_list = g_key_file_get_string_list (state_file,
@@ -195,35 +381,20 @@ ev_application_load_session (EvApplication *application)
 	return TRUE;
 }
 
+#ifdef WITH_SMCLIENT
+
 static void
 smclient_save_state_cb (EggSMClient   *client,
 			GKeyFile      *state_file,
 			EvApplication *application)
 {
-	GList *windows, *l;
-	gint i;
-	const gchar **uri_list;
-	const gchar *empty = "empty-window";
+	GList *windows;
 
 	windows = ev_application_get_windows (application);
-	if (!windows)
-		return;
-
-	uri_list = g_new (const gchar *, g_list_length (windows));
-	for (l = windows, i = 0; l != NULL; l = g_list_next (l), i++) {
-		EvWindow *window = EV_WINDOW (l->data);
-
-		if (ev_window_is_empty (window))
-			uri_list[i] = empty;
-		else
-			uri_list[i] = ev_window_get_uri (window);
+	if (windows) {
+		save_session (application, windows, state_file);
+		g_list_free (windows);
 	}
-	g_key_file_set_string_list (state_file,
-				    "Evince",
-				    "documents", 
-				    (const char **)uri_list,
-				    i);
-	g_free (uri_list);
 }
 
 static void
@@ -233,9 +404,17 @@ smclient_quit_cb (EggSMClient   *client,
 	ev_application_shutdown (application);
 }
 
+#endif /* WITH_SMCLIENT */
+
 static void
 ev_application_init_session (EvApplication *application)
 {
+#ifdef ENABLE_DBUS
+	application->crashed_file = g_build_filename (application->dot_dir,
+						      "evince-crashed", NULL);
+#endif
+
+#ifdef WITH_SMCLIENT
 	application->smclient = egg_sm_client_get ();
 	g_signal_connect (application->smclient, "save_state",
 			  G_CALLBACK (smclient_save_state_cb),
@@ -243,6 +422,7 @@ ev_application_init_session (EvApplication *application)
 	g_signal_connect (application->smclient, "quit",
 			  G_CALLBACK (smclient_quit_cb),
 			  application);
+#endif
 }
 
 /**
@@ -383,48 +563,6 @@ get_find_string_from_args (GHashTable *args)
 	return value ? g_value_get_string (value) : NULL;
 }
 
-/**
- * get_unlink_temp_file_from_args:
- * @args: a #GHashTable with data passed to the application.
- *
- * It does look if the unlink-temp-file option has been passed from the command
- * line returning it's boolean representation, otherwise it does return %FALSE.
- *
- * Returns: the boolean representation of the unlink-temp-file value or %FALSE
- *          in other case.
- */
-static gboolean
-get_unlink_temp_file_from_args (GHashTable *args)
-{
-	gboolean unlink_temp_file = FALSE;
-	GValue  *value = NULL;
-
-	g_assert (args != NULL);
-
-	value = g_hash_table_lookup (args, "unlink-temp-file");
-	if (value) {
-		unlink_temp_file = g_value_get_boolean (value);
-	}
-	
-	return unlink_temp_file;
-}
-
-static const gchar *
-get_print_settings_from_args (GHashTable *args)
-{
-	const gchar *print_settings = NULL;
-	GValue      *value = NULL;
-
-	g_assert (args != NULL);
-
-	value = g_hash_table_lookup (args, "print-settings");
-	if (value) {
-		print_settings = g_value_get_string (value);
-	}
-
-	return print_settings;
-}
-
 /**
  * ev_application_open_window:
  * @application: The instance of the application.
@@ -452,9 +590,16 @@ ev_application_open_window (EvApplication  *application,
 	}
 	
 	if (screen) {
+		ev_stock_icons_set_screen (screen);
 		gtk_window_set_screen (GTK_WINDOW (new_window), screen);
 	}
-	ev_application_add_icon_path_for_screen (screen);
+
+#ifdef ENABLE_DBUS
+	ev_application_save_session_crashed (application);
+	g_signal_connect_swapped (new_window, "destroy",
+				  G_CALLBACK (save_session_crashed_in_idle),
+				  application);
+#endif
 
 	if (!GTK_WIDGET_REALIZED (new_window))
 		gtk_widget_realize (new_window);
@@ -543,37 +688,6 @@ ev_application_get_uri_window (EvApplication *application, const char *uri)
 	return uri_window;
 }
 
-static void
-ev_application_add_icon_path_for_screen (GdkScreen *screen)
-{
-	GtkIconTheme *icon_theme;
-
-	icon_theme = screen ? gtk_icon_theme_get_for_screen (screen) : gtk_icon_theme_get_default ();
-	if (icon_theme) {
-		gchar **path = NULL;
-		gint    n_paths;
-		gint    i;
-		gchar  *ev_icons_path;
-
-		/* GtkIconTheme will then look in Evince custom hicolor dir
-		 * for icons as well as the standard search paths
-		 */
-		ev_icons_path = g_build_filename (DATADIR, "icons", NULL);
-		gtk_icon_theme_get_search_path (icon_theme, &path, &n_paths);
-		for (i = n_paths - 1; i >= 0; i--) {
-			if (g_ascii_strcasecmp (ev_icons_path, path[i]) == 0)
-				break;
-		}
-
-		if (i < 0)
-			gtk_icon_theme_append_search_path (icon_theme,
-							   ev_icons_path);
-
-		g_free (ev_icons_path);
-		g_strfreev (path);
-	}	
-}
-
 /**
  * ev_application_open_uri_at_dest:
  * @application: The instance of the application.
@@ -581,7 +695,6 @@ ev_application_add_icon_path_for_screen (GdkScreen *screen)
  * @screen: Thee screen where the link will be shown.
  * @dest: The #EvLinkDest of the document.
  * @mode: The run mode of the window.
- * @unlink_temp_file: The unlink_temp_file option value.
  * @timestamp: Current time value.
  */
 void
@@ -591,16 +704,12 @@ ev_application_open_uri_at_dest (EvApplication  *application,
 				 EvLinkDest     *dest,
 				 EvWindowRunMode mode,
 				 const gchar    *search_string,
-				 gboolean        unlink_temp_file,
-				 const gchar    *print_settings, 
 				 guint           timestamp)
 {
 	EvWindow *new_window;
 
 	g_return_if_fail (uri != NULL);
 	
-	ev_application_add_icon_path_for_screen (screen);
-
 	new_window = ev_application_get_uri_window (application, uri);
 	
 	if (new_window == NULL) {
@@ -611,13 +720,21 @@ ev_application_open_uri_at_dest (EvApplication  *application,
 		new_window = EV_WINDOW (ev_window_new ());
 	}
 
-	if (screen)
+	if (screen) {
+		ev_stock_icons_set_screen (screen);
 		gtk_window_set_screen (GTK_WINDOW (new_window), screen);
+	}
 
 	/* We need to load uri before showing the window, so
 	   we can restore window size without flickering */	
-	ev_window_open_uri (new_window, uri, dest, mode, search_string, 
-			    unlink_temp_file, print_settings);
+	ev_window_open_uri (new_window, uri, dest, mode, search_string);
+
+#ifdef ENABLE_DBUS
+	ev_application_save_session_crashed (application);
+	g_signal_connect_swapped (new_window, "destroy",
+				  G_CALLBACK (save_session_crashed_in_idle),
+				  application);
+#endif
 
 	if (!GTK_WIDGET_REALIZED (GTK_WIDGET (new_window)))
 		gtk_widget_realize (GTK_WIDGET (new_window));
@@ -655,8 +772,6 @@ ev_application_open_uri (EvApplication  *application,
 	EvLinkDest      *dest = NULL;
 	EvWindowRunMode  mode = EV_WINDOW_MODE_NORMAL;
 	const gchar     *search_string = NULL;
-	gboolean         unlink_temp_file = FALSE;
-	const gchar     *print_settings = NULL;
 	GdkScreen       *screen = NULL;
 
 	if (args) {
@@ -664,15 +779,11 @@ ev_application_open_uri (EvApplication  *application,
 		dest = get_destination_from_args (args);
 		mode = get_window_run_mode_from_args (args);
 		search_string = get_find_string_from_args (args);
-		unlink_temp_file = (mode == EV_WINDOW_MODE_PREVIEW &&
-				    get_unlink_temp_file_from_args (args));
-		print_settings = get_print_settings_from_args (args);
 	}
 	
 	ev_application_open_uri_at_dest (application, uri, screen,
 					 dest, mode, search_string,
-					 unlink_temp_file,
-					 print_settings, timestamp);
+					 timestamp);
 
 	if (dest)
 		g_object_unref (dest);
@@ -691,13 +802,21 @@ ev_application_open_uri_list (EvApplication *application,
 	for (l = uri_list; l != NULL; l = l->next) {
 		ev_application_open_uri_at_dest (application, (char *)l->data,
 						 screen, NULL, 0, NULL,
-						 FALSE, NULL, timestamp);
+						 timestamp);
 	}
 }
 
 void
 ev_application_shutdown (EvApplication *application)
 {
+#ifdef ENABLE_DBUS
+	if (application->crashed_file) {
+		ev_application_save_session_crashed (application);
+		g_free (application->crashed_file);
+		application->crashed_file = NULL;
+	}
+#endif
+
 	if (application->accel_map_file) {
 		gtk_accel_map_save (application->accel_map_file);
 		g_free (application->accel_map_file);
@@ -739,8 +858,12 @@ ev_application_shutdown (EvApplication *application)
 
         g_free (application->dot_dir);
         application->dot_dir = NULL;
-	g_free (application->last_chooser_uri);
-        application->last_chooser_uri = NULL;
+        g_free (application->data_dir);
+        application->data_dir = NULL;
+	g_free (application->filechooser_open_uri);
+        application->filechooser_open_uri = NULL;
+	g_free (application->filechooser_save_uri);
+	application->filechooser_save_uri = NULL;
 
 	g_object_unref (application);
         instance = NULL;
@@ -758,8 +881,7 @@ ev_application_init (EvApplication *ev_application)
 {
 	gint i;
 	const gchar *home_dir;
-	
-	ev_application_init_session (ev_application);
+	gchar *toolbar_path;
 
         ev_application->dot_dir = g_build_filename (g_get_home_dir (),
                                                     ".gnome2",
@@ -770,11 +892,25 @@ ev_application_init (EvApplication *ev_application)
         if (!ev_dir_ensure_exists (ev_application->dot_dir, 0700))
                 exit (1);
 
+#ifdef G_OS_WIN32
+{
+	gchar *dir;
+
+	dir = g_win32_get_package_installation_directory_of_module (NULL);
+	ev_application->data_dir = g_build_filename (dir, "share", "evince", NULL);
+	g_free (dir);
+}
+#else
+	ev_application->data_dir = g_strdup (DATADIR);
+#endif
+
+	ev_application_init_session (ev_application);
+
 	home_dir = g_get_home_dir ();
 	if (home_dir) {
 		ev_application->accel_map_file = g_build_filename (home_dir,
 								   ".gnome2",
-								   "accels"
+								   "accels",
 								   "evince",
 								   NULL);
 		gtk_accel_map_load (ev_application->accel_map_file);
@@ -785,14 +921,17 @@ ev_application_init (EvApplication *ev_application)
 	ev_application->toolbars_file = g_build_filename
 			(ev_application->dot_dir, "evince_toolbar.xml", NULL);
 
+	toolbar_path = g_build_filename (ev_application->data_dir,
+					 "evince-toolbar.xml", NULL);
 	egg_toolbars_model_load_names (ev_application->toolbars_model,
-				       DATADIR "/evince-toolbar.xml");
+				       toolbar_path);
 
 	if (!egg_toolbars_model_load_toolbars (ev_application->toolbars_model,
 					       ev_application->toolbars_file)) {
 		egg_toolbars_model_load_toolbars (ev_application->toolbars_model,
-						  DATADIR"/evince-toolbar.xml");
+						  toolbar_path);
 	}
+	g_free (toolbar_path);
 
 	/* Open item doesn't exist anymore,
 	 * convert it to OpenRecent for compatibility
@@ -877,16 +1016,32 @@ ev_application_save_toolbars_model (EvApplication *application)
 }
 
 void
-ev_application_set_chooser_uri (EvApplication *application, const gchar *uri)
+ev_application_set_filechooser_uri (EvApplication       *application,
+				    GtkFileChooserAction action,
+				    const gchar         *uri)
 {
-	g_free (application->last_chooser_uri);
-	application->last_chooser_uri = g_strdup (uri);
+	if (action == GTK_FILE_CHOOSER_ACTION_OPEN) {
+		g_free (application->filechooser_open_uri);
+		application->filechooser_open_uri = g_strdup (uri);
+	} else if (action == GTK_FILE_CHOOSER_ACTION_SAVE) {
+		g_free (application->filechooser_save_uri);
+		application->filechooser_save_uri = g_strdup (uri);
+	}
 }
 
 const gchar *
-ev_application_get_chooser_uri (EvApplication *application)
+ev_application_get_filechooser_uri (EvApplication       *application,
+				    GtkFileChooserAction action)
 {
-	return application->last_chooser_uri;
+	if (action == GTK_FILE_CHOOSER_ACTION_OPEN) {
+		if (application->filechooser_open_uri)
+			return application->filechooser_open_uri;
+	} else if (action == GTK_FILE_CHOOSER_ACTION_SAVE) {
+		if (application->filechooser_save_uri)
+			return application->filechooser_save_uri;
+	}
+
+	return NULL;
 }
 
 void
@@ -1048,3 +1203,9 @@ ev_application_get_dot_dir (EvApplication   *application)
 {
 	return application->dot_dir;
 }
+
+const gchar *
+ev_application_get_data_dir (EvApplication   *application)
+{
+	return application->data_dir;
+}
